@@ -1,5 +1,5 @@
 #interface objects
-from .InterfaceObjects import ASG,StopCondition
+from .InterfaceObjects import ASG,StopCondition,ASGAlgorithmData
 #import helper functions
 from .HelperFunctions import *
 
@@ -8,10 +8,7 @@ from .HelperFunctions import *
 #   ASG extension
 #
 #       TODO:
-#               - Discuss: Do we need to provide the rules in the getASG()
-#               - Discuss: Do we need to provide the rules in the setASG()
-#               - Discuss: Can we assume that the assembly system is in place when performing setASG()
-#               - Discuss: Do we need seperate functions to UPDATE and ADD DFA rules?
+#               -
 #
 # -----------------------------------------------------------------------------------------------------------
 
@@ -49,6 +46,18 @@ def getASG(AssemblySystemName,model,KBPath):
                         InterfaceObject.Generator=[ASG_temp.hasMultiPartGenerator.hasName, 'MultiPartGenerator']
                 except:
                     x = "No MultiPartGenerator available"
+                try:
+                        InterfaceObject.Generator=[ASG_temp.hasBatchgenerator.hasName, 'BatchGenerator']
+                except:
+                    x = "No BatchGenerator available"
+                try:
+                        InterfaceObject.Generator=[ASG_temp.hasForcegenerator.hasName, 'ForceGenerator']
+                except:
+                    x = "No ForceGenerator available"
+                try:
+                        InterfaceObject.Generator=[ASG_temp.hasPredefinedGenerator.hasName, 'PredefinedGenerator']
+                except:
+                    x = "No PredefinedGenerator available"
 
                 # try-catch structure to fetch the ASG SELECTOR
                 try:
@@ -56,9 +65,15 @@ def getASG(AssemblySystemName,model,KBPath):
                 except:
                     x = "No Default selector available"
                 try:
-                        InterfaceObject.Selector=[ASG_temp.hasRuleSelector.hasName, 'RuleSelector']             #TODO: discuss if we need to fetch the rules
+                        InterfaceObject.Selector=[ASG_temp.hasRuleSelector.hasName, 'RuleSelector']
+                        # TODO: [DISCUSS] Do we need to fetch the rules?
                 except:
                     x = "No Rule selector available"
+                try:
+                        InterfaceObject.Selector=[ASG_temp.hasManualselector.hasName, 'ManualSelector']
+                        # TODO: [DISCUSS] Do we need to fetch the operations?
+                except:
+                    x = "No Manual selector available"
 
                 # try-catch structure to fetch the ASG EVALUATOR
                 try:
@@ -127,6 +142,18 @@ def updateASG(AssemblySystemName,interfaceObject,model,KBPath):
                     assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasMultiPartGenerator.hasName = interfaceObject.Generator[0]
                 except:
                     x = "No MultiPartGenerator available"
+                try:
+                    assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasBatchgenerator.hasName = interfaceObject.Generator[0]
+                except:
+                    x = "No BatchGenerator available"
+                try:
+                    assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasForcegenerator.hasName = interfaceObject.Generator[0]
+                except:
+                    x = "No ForceGenerator available"
+                try:
+                    assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasPredefinedGenerator.hasName = interfaceObject.Generator[0]
+                except:
+                    x = "No PredefinedGenerator available"
 
                 # -- ASG SELECTOR --
                 try:
@@ -135,9 +162,14 @@ def updateASG(AssemblySystemName,interfaceObject,model,KBPath):
                     x = "No Default selector available"
                 try:
                     assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasRuleSelector.hasName = interfaceObject.Selector[0]
-                    #TODO: discuss if we need to update the rules or via dedicated function!
+                    # TODO: [DISCUSS] Do we need to update the rules?
                 except:
                     x = "No Rule selector available"
+                try:
+                    assemblySystem.hasAssemblyAlgorithmConfigurationhasSinglePartGenerator.hasManualselector.hasName = interfaceObject.Selector[0]
+                    # TODO: [DISCUSS] Do we need to update the operations?
+                except:
+                    x = "No Manual selector available"
 
                 # -- ASG EVALUATOR --
                 try:
@@ -217,6 +249,39 @@ def setASG( AssemblySystemName, InterfaceObject,model,KBPath,path_ecore,MM,API):
             g_.hasName = InterfaceObject.Generator[0]
             ASG_.hasMultiPartGenerator = g_
 
+    if 'BatchGenerator' in InterfaceObject.Generator[1]:
+        # -- ASG GENERATOR --
+        try:
+            ASG_.hasBatchgenerator.hasName = InterfaceObject.Generator[0]
+        except:
+            x = "No BatchGenerator available"
+            # -- create new class if not existing --
+            g_ = API.create_noPlatformRoot("BatchGenerator")
+            g_.hasName = InterfaceObject.Generator[0]
+            ASG_.hasBatchgenerator = g_
+
+    if 'ForceGenerator' in InterfaceObject.Generator[1]:
+        # -- ASG GENERATOR --
+        try:
+            ASG_.hasForcegenerator.hasName = InterfaceObject.Generator[0]
+        except:
+            x = "No ForceGenerator available"
+            # -- create new class if not existing --
+            g_ = API.create_noPlatformRoot("ForceGenerator")
+            g_.hasName = InterfaceObject.Generator[0]
+            ASG_.hasForcegenerator = g_
+
+    if 'PredefinedGenerator' in InterfaceObject.Generator[1]:
+        # -- ASG GENERATOR --
+        try:
+            ASG_.hasPredefinedGenerator.hasName = InterfaceObject.Generator[0]
+        except:
+            x = "No PredefinedGenerator available"
+            # -- create new class if not existing --
+            g_ = API.create_noPlatformRoot("PredefinedGenerator")
+            g_.hasName = InterfaceObject.Generator[0]
+            ASG_.hasPredefinedGenerator = g_
+
     if 'DefaultSelector' in InterfaceObject.Selector[1]:
         # -- ASG SELECTOR --
         try:
@@ -239,6 +304,17 @@ def setASG( AssemblySystemName, InterfaceObject,model,KBPath,path_ecore,MM,API):
             g_.hasName = InterfaceObject.Selector[0]
             ASG_.hasRuleSelector = g_
 
+    if 'ManualSelector' in InterfaceObject.Selector[1]:
+        # -- ASG SELECTOR --
+        try:
+            ASG_.hasManualselector.hasName = InterfaceObject.Selector[0]
+        except:
+            x = "No ManualSelector available"
+            # -- create new class if not existing --
+            g_ = API.create_noPlatformRoot("ManualSelector")
+            g_.hasName = InterfaceObject.Selector[0]
+            ASG_.hasManualselector = g_
+
     if 'UniformEvaluator' in InterfaceObject.Evaluator[1]:
         # -- ASG EVALUATOR --
         try:
@@ -276,4 +352,107 @@ def setASG( AssemblySystemName, InterfaceObject,model,KBPath,path_ecore,MM,API):
 
     AS.hasAssemblyAlgorithmConfiguration = ASG_
 
-    updateKBv6(KBPath,model)
\ No newline at end of file
+    updateKBv6(KBPath,model)
+
+def getASGAlgorithmData(AssemblySystemName,model):
+    """
+              Function to fetch the Assembly Sequence Generation (ASG) data using the name as identifier.
+              The function returns an interface block of the ASG
+
+              :param string AssemblySystemName: Name identifier of the assemlbySystem
+              :param object model: Metamodel instance model
+
+              :return object ASGAlgorithmData: Interface object of the ASGData (-1 = error)
+        """
+    found = 0
+    InterfaceObject = ASG(None)  # interface object placeholder
+    if len(model.includesAssemblySystem.items) != 0:
+        # match the assemblySystem by Name
+        for assemblySystem in model.includesAssemblySystem.items:
+            if AssemblySystemName == assemblySystem.hasName:
+                found = 1
+                # fetch ASG and setup ASG return
+                if assemblySystem.hasAssemblyAlgorithmData is not None:
+                    InterfaceObject = ASGAlgorithmData(Score=assemblySystem.hasAssemblyAlgorithmData.hasScore,Feasible=assemblySystem.hasAssemblyAlgorithmData.isFeasible,UnexploredOptions=assemblySystem.hasAssemblyAlgorithmData.unexploredOptions,Priority=assemblySystem.hasAssemblyAlgorithmData.hasPriority)
+                else:
+                    InterfaceObject = -1
+            else:
+                x = 1
+
+        if found:
+            return InterfaceObject
+        else:
+            return -1
+    else:
+        return -1  # no ASG data
+
+def updateASGAlgorithmData(AssemblySystemName,interfaceObject, model, KBPath):
+    """
+              Function to update the  Assembly Sequence Generation (ASG) data using the name as identifier.
+              The ASG interface object is used to interface with the function.
+              The function returns whether or not the function is performed correctly
+
+              :param string AssemblySystemName: Name identifier of the assemlbySystem
+              :param object ASG data Model: Interface object of the ASG
+              :param object model: Metamodel instance model
+              :param string KBPath: Absolute path to the metamodel instance model
+
+              :return int Error: -1 = error, 1= function performed correcly
+        """
+    found = 0
+    if len(model.includesAssemblySystem.items) != 0:
+        # match the assemblySystem by Name
+        for assemblySystem in model.includesAssemblySystem.items:
+            if AssemblySystemName == assemblySystem.hasName:
+                # fetch ASG data and perform KB update
+                if assemblySystem.hasAssemblyAlgorithmConfiguration is not None:
+                    found = 1
+                    assemblySystem.hasAssemblyAlgorithmData.hasScore = interfaceObject.Score
+                    assemblySystem.hasAssemblyAlgorithmData.isFeasible = interfaceObject.Feasible   #TODO: Boolean does not get serialized!
+                    assemblySystem.hasAssemblyAlgorithmData.unexploredOptions = interfaceObject.UnexploredOptions
+                    assemblySystem.hasAssemblyAlgorithmData.hasPriority = interfaceObject.Priority
+            else:
+                x = 1
+
+        if found:
+            updateKBv6(KBPath, model)
+            return 1
+        else:
+            return -1
+    else:
+        return -1  # no ASG data
+
+def setASGAlgorithmData(AssemblySystemName, interfaceObject,model,KBPath,path_ecore,MM,API):
+    """
+             Function to set a Assembly Sequence Generation (ASG) and corresponding data, to a new or existing Assembly System
+             The performance interface object is used to interface with the function.
+
+
+             :param string AssemblySystemName: Name of the Assembly system
+             :param object ASG data Model: Interface object of the ASG model
+             :param object model: Metamodel instance model
+             :param string KBPath: Absolute path to the metamodel instance model
+             :param string KBPath: Absolute path to the metamodel ecore model
+
+             :return -
+       """
+    AS = None
+    if len(model.includesAssemblySystem.items) != 0:
+        # match the assemblySystem by Name
+        for assemblySystem in model.includesAssemblySystem.items:
+            if AssemblySystemName == assemblySystem.hasName:
+                AS = assemblySystem
+    if AS is None:
+        # -- create new AssemblySystem --
+        AS = API.create_connected("AssemblySystem")
+        AS.hasName = AssemblySystemName
+
+    # -- create new ASG data --
+    ASGData = API.create_noPlatformRoot("AssemblyAlgorithmData")
+    ASGData.hasScore = interfaceObject.Score
+    ASGData.isFeasible = interfaceObject.Feasible  # TODO: Boolean does not get serialized!
+    ASGData.unexploredOptions = interfaceObject.UnexploredOptions
+    ASGData.hasPriority = interfaceObject.Priority
+
+    AS.hasAssemblyAlgorithmData = ASGData
+    updateKBv6(KBPath, model)
\ No newline at end of file
