@@ -73,6 +73,43 @@ class ASG():
         """
         return -1 #yaml.dump(self,indent=4)
 
+class ASGAlgorithmData():
+    """
+            ASGAlgorithmData: Class representing the (performance)data for a given ASG algorithm
+
+             :param double Score: Score of the ASG algorithm
+             :param boolean Feasible: is feasible indication of the ASG algorithm
+             :param string UnexploredOptions: Unexplored options of the ASG algorithm
+             :param double Priority: Priority of the ASG algorithm
+
+        """
+    def __init__(self,JSONDescriptor=None,Score=0.0,Feasible=False,UnexploredOptions="",Priority=1.0,DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Score = Score
+            self.Feasible = Feasible
+            self.UnexploredOptions = UnexploredOptions
+            self.Priority = Priority
+        else:
+            self.json2object(JSONDescriptor)
+            x=10
+
+    def json2object(self,jsonDescriptor):
+        """
+             Function to generate a ASG algorithm data object from a JSON file
+
+             :param string jsonDescriptor: absolute path to the json file for the DFA rule
+
+        """
+        #--interpret JSON file--
+        x=1
+
+    def object2json(self):
+        """
+            Function to generate a json file from the ASG algorithm dta
+        """
+        return json.dumps(self, default=lambda o: o.__dict__,indent=4)
+
 class DFARule():
     """
         DFARule: Class representing the DFA rule
@@ -170,7 +207,7 @@ class PerformanceModel():
         :param objectlist ObjectiveList: list of optimization objectives
         :param objectlist ConstraintList: list of optimization constraints
     """
-    def __init__(self, JSONDescriptor=None, Name="", Description="",OptimizationMethod=[],MethodPerformance=[],RAWResults=[],parameterlist = [],StopConditions = [],ObjectiveList=[],ConstraintList=[],interpretedResults=[],DEBUG=True):
+    def __init__(self, JSONDescriptor=None, Name="", Description="",OptimizationMethod=None,MethodPerformance=[],RAWResults=[],parameterlist = [],StopConditions = [],ObjectiveList=[],ConstraintList=[],interpretedResults=[],DEBUG=True):
         self.DEBUG = DEBUG
         if JSONDescriptor is None:
             self.Name = Name
@@ -222,7 +259,7 @@ class PerformanceModel():
         for var in jsonObject["parameterList"]:
             pList = []
             for p in var["parameterList"]:
-                par = Parameter(Name=p["Name"], Description=p["Description"],Key=p["Key"],GUID=p["GUID"], Value=p["Value"], Stopcriteria=p["StopCriteria"])
+                par = Parameter(Name=p["Name"], Description=p["Description"],Key=p["Key"],GUID=p["GUID"], Value=p["Value"], Stopcriteria=p["StopCriteria"],Unit=p["Unit"])
                 pList.append(par)
             decisionvariable = DecisionVariable(Name=var["Name"], Description=var["Description"], InitialValue=var["InitialValue"],MinValue=var["MinValue"], MaxValue=var["MaxValue"], Optimum=var["Optimum"],Resolution = var["Resolution"],parameters=pList)
             self.parameterList.append(decisionvariable)
@@ -230,13 +267,27 @@ class PerformanceModel():
         # Objectives
         self.ObjectiveList = []
         for objective in jsonObject["ObjectiveList"]:
-            obj_ = Objective(Name=objective["Name"], Description=objective["Description"], ObjectiveOption=objective["ObjectiveOption"])
+            if len(objective["ObjectiveTerm"])==3:
+                term = Term(None,Value=[objective["ObjectiveTerm"][0],objective["ObjectiveTerm"][1],objective["ObjectiveTerm"][2]])
+            else:
+                term = Term(None, Value=objective["ObjectiveTerm"][0])  # if string or float
+            obj_ = Objective(Name=objective["Name"], Description=objective["Description"], ObjectiveOption=objective["ObjectiveOption"],ObjectiveTerm=term)
             self.ObjectiveList.append(obj_)
 
         # Constraints
         self.ConstraintList = []
         for cnstr in jsonObject["ConstraintList"]:
-            cnstr_ = Constraint(Name=cnstr["Name"], Description=cnstr["Description"],Expression=cnstr["Expression"])
+            if len(cnstr["Expression"][0]) == 4:
+                Lterm = Term(None,Value=[cnstr["Expression"][0][1],cnstr["Expression"][0][2],cnstr["Expression"][0][3]])
+            else:
+                Lterm = Term(None, Value=cnstr["Expression"][0][1])  # if string or float
+            if len(cnstr["Expression"][1]) == 4:
+                Rterm = Term(None,Value=[cnstr["Expression"][1][1], cnstr["Expression"][1][2], cnstr["Expression"][1][3]])
+            else:
+                Rterm = Term(None, Value=cnstr["Expression"][1][1])  # if string or float
+            operator = cnstr["Expression"][2]
+            expression = Expression(JSONDescriptor=None,LeftTerm=Lterm,ExpressionOperator=operator,RightTerm=Rterm)
+            cnstr_ = Constraint(Name=cnstr["Name"], Description=cnstr["Description"],Expression=expression)
             self.ConstraintList.append(cnstr_)
 
     def clean(self):
@@ -281,9 +332,10 @@ class Parameter():
              :param string Key: Key of the parameter
              :param string GUID: Global unique identifier of the parameter
              :param float Value: Value of the parameter
+             :param string Unit: Unit of measurement of the parameter
              :param enum StopCriteria: StopCriteria of the parameter
     """
-    def __init__(self, JSONDescriptor=None, Name="", Description="",Key="",GUID="", Value=0.0, Stopcriteria="maxDepth=0", DEBUG=True):
+    def __init__(self, JSONDescriptor=None, Name="", Description="",Key="",GUID="", Value=0.0, Stopcriteria="maxDepth=0",Unit="", DEBUG=True):
         self.DEBUG = DEBUG
         if JSONDescriptor is None:
             self.Name = Name
@@ -292,6 +344,7 @@ class Parameter():
             self.GUID = GUID
             self.Value = Value
             self.StopCriteria = Stopcriteria
+            self.Unit=Unit
         else:
             # TODO: import json descriptor to setup object
             x = 1
@@ -304,11 +357,12 @@ class Objective():
          :param string Description: Description of the Objective
          :param enum ObjectiveOption: ObjectiveOption of the Objective
     """
-    def __init__(self, JSONDescriptor=None, Name="", Description="",ObjectiveOption="Minimize", DEBUG=True):
+    def __init__(self, JSONDescriptor=None, Name="", Description="",ObjectiveTerm=None,ObjectiveOption="Minimize", DEBUG=True):
         self.DEBUG = DEBUG
         if JSONDescriptor is None:
             self.Name = Name
             self.Description = Description
+            self.ObjectiveTerm = ObjectiveTerm      #Added 18/10/2022 meeting Pavel
             self.ObjectiveOption = ObjectiveOption
 
         else:
@@ -323,7 +377,7 @@ class Constraint():
          :param string Description: Description of the constraint
          :param list Expression: Expression of the constraint
     """
-    def __init__(self, JSONDescriptor=None, Name="", Description="",Expression=[], DEBUG=True):
+    def __init__(self, JSONDescriptor=None, Name="", Description="",Expression=None, DEBUG=True):
         self.DEBUG = DEBUG
         if JSONDescriptor is None:
             self.Name = Name
@@ -334,6 +388,81 @@ class Constraint():
             # TODO: import json descriptor to setup object
             x = 10
 
+#----------------------Added 18/10/2022 meeting Pavel--------------------------
+class Term():
+    """
+        Term: Class representing a Term of an expression
+
+         :param string|float|object Value: Value of the term (different datatypes: string|float|object)
+    """
+    def __init__(self, JSONDescriptor=None, Value=None, DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Value = Value
+
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 10
+
+class Expression():
+    """
+        Expression: Class representing an expression
+
+         :param object LeftTerm: LeftTerm of the expression (can also be an expression)
+         :param string ExpressionOperator: Expression operator value e.g. "+"
+         :param object RightTerm: RightTerm of the expression (can also be an expression)
+    """
+    def __init__(self, JSONDescriptor=None, LeftTerm=None, ExpressionOperator="+",RightTerm=None, DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.LeftTerm = LeftTerm
+            self.ExpressionOperator = ExpressionOperator
+            self.RightTerm = RightTerm
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 10
+
+class OptimizationMethod():
+    """
+        Term: Class representing an optimization method
+
+         :param string Name: Name of the OptimizationMethod
+         :param string Description: Description of the OptimizationMethod
+         :param string OptimizationAlgorithmClass: Class of the OptimizationMethod ("Heuristic"|"Deterministic"|"Metaheuristic")
+         :param list OptionList: List of options for the OptimizationMethod
+    """
+    def __init__(self, JSONDescriptor=None, Name="", Description="",OptimizationAlgorithmClass="Heuristic",OptionList=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Description = Description
+            self.OptimizationAlgorithmClass = OptimizationAlgorithmClass
+            self.OptionList = OptionList
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 10
+
+class Option():
+    """
+        Option: Class representing an option
+
+         :param string Name: Name of the Option
+         :param string|boolean|float|object|list|[hasX,hasY,hasZ]|("AddingIsWorse"|"ConstantSurfaceOnly") Value: Value of the term (different datatypes: string|boolean|float|object|list|[hasX,hasY,hasZ]|("AddingIsWorse"|"ConstantSurfaceOnly"))
+    """
+    def __init__(self, JSONDescriptor=None, Name="",Value=None, DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Value = Value
+
+
+        else:
+            # TODO: import json descriptor to setup object
+            x = 10
+#------------------------------------------------------------------------------
 class DecisionVariable():
     """
         DecisionVariable: Class representing the optimization Objective
@@ -345,9 +474,10 @@ class DecisionVariable():
          :param float MaxValue: MaxValue of the DecisionVariable
          :param float Optimum: Optimum of the DecisionVariable
          :param float Resolution: Resolution of the DecisionVariable
+         :param string Unit: Unit of measurement of the DecisionVariable
          :param objectlist parameterList: list of linked parameters
     """
-    def __init__(self, JSONDescriptor=None, Name="", Description="",InitialValue=0.0,MinValue=0.0, MaxValue=0.0, Optimum=0.0,Resolution = 0.0,parameters=[], DEBUG=True):
+    def __init__(self, JSONDescriptor=None, Name="", Description="",InitialValue=0.0,MinValue=0.0, MaxValue=0.0, Optimum=0.0,Resolution = 0.0,parameters=[],Unit="", DEBUG=True):
         self.DEBUG = DEBUG
         if JSONDescriptor is None:
             self.Name = Name
@@ -358,6 +488,7 @@ class DecisionVariable():
             self.Optimum = Optimum
             self.Resolution = Resolution
             self.parameterList = parameters
+            self.Unit = Unit
         else:
             # TODO: import json descriptor to setup object
             x = 1
@@ -399,7 +530,7 @@ class Product():
 
         self.parameterList = []
         for par in jsonObject['parameterList']:
-            p = Parameter(None, Name=par['Name'], Description=par['Description'],Key=par['Key'], GUID=par['GUID'],Value=par['Value'],Stopcriteria=par['StopCriteria'])
+            p = Parameter(None, Name=par['Name'], Description=par['Description'],Key=par['Key'], GUID=par['GUID'],Value=par['Value'],Stopcriteria=par['StopCriteria'],Unit=par["Unit"])
             self.parameterList.append(p)
         #TODO: discuss if we need to add the rules at once?
 
@@ -419,8 +550,9 @@ class ProductPart():
          :param objectlist parameterList: list of linked parameters
          :param objectlist partList: list of linked product parts
          :param list material: list of the material properties
+         :param boolean isFastner: identification if productpart is fastner
     """
-    def __init__(self, JSONDescriptor=None, Name="", Description="",ProductType='',Quantity=0,parameters = [],partList=[],material = [], DEBUG=True):
+    def __init__(self, JSONDescriptor=None, Name="", Description="",ProductType='',Quantity=0,parameters = [],partList=[],material = [],isFastner=False, DEBUG=True):
         self.DEBUG = DEBUG
         if JSONDescriptor is None:
             self.Name = Name
@@ -430,6 +562,7 @@ class ProductPart():
             self.parameterList = parameters
             self.partList = partList
             self.material = material    #TODO: Stiffness not yet correct
+            self.isFastner = isFastner
 
         else:
             self.json2object(JSONDescriptor)
@@ -452,8 +585,12 @@ class ProductPart():
         self.ProductType = jsonObject['ProductType']
         self.Quantity = jsonObject['Quantity']
         self.parameterList = []
+        try:
+            self.isFastner = jsonObject['isFastner']
+        except:
+            self.isFastner = False
         for par in jsonObject['parameterList']:
-            p = Parameter(None, Name=par['Name'], Description=par['Description'], Key=par['Key'], GUID=par['GUID'],Value=par['Value'], Stopcriteria=par['StopCriteria'])
+            p = Parameter(None, Name=par['Name'], Description=par['Description'], Key=par['Key'], GUID=par['GUID'],Value=par['Value'], Stopcriteria=par['StopCriteria'],Unit=par["Unit"])
             self.parameterList.append(p)
         self.partList = []
         for subPart in jsonObject['partList']:
@@ -475,7 +612,7 @@ class ProductPart():
 
         part_temp.parameterList = []
         for par in subPart['parameterList']:
-            p = Parameter(None, Name=par['Name'], Description=par['Description'], Key=par['Key'], GUID=par['GUID'], Value=par['Value'], Stopcriteria=par['StopCriteria'])
+            p = Parameter(None, Name=par['Name'], Description=par['Description'], Key=par['Key'], GUID=par['GUID'], Value=par['Value'], Stopcriteria=par['StopCriteria'],Unit=par["Unit"])
             part_temp.parameterList.append(p)
         part_temp.partList = []
         for subPart in subPart['partList']:
@@ -721,6 +858,263 @@ class Operation():       #
         """
         return json.dumps(self, default=lambda o: o.__dict__, indent=4)
 
+class Material():       #
+    """
+        Material: Class representing the material
+
+         :param string Name: Name of the DomainVariable
+         :param Double Density: Density of the material
+         :param Double Cost: Cost of the material
+         :param Double Damping: Damping of the material
+         :param Double Transparency: Transparency of the material
+         :param list Stiffness: Stiffness matrix of the material
+
+
+    """
+
+    def __init__(self, JSONDescriptor=None, Name="", Density=7.89,Cost=1.0, Damping=88.5,Transparency=90.0,Stiffness=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Density = Density
+            self.Cost = Cost
+            self.Damping = Damping
+            self.Transparency = Transparency
+            self.Stiffness = Stiffness
+
+        else:
+            self.json2object(JSONDescriptor)
+            x = 10
+
+    def json2object(self, jsonDescriptor):
+        """
+             Function to generate a Material object from a JSON file
+
+             :param string jsonDescriptor: absolute path to the json file for the Material
+
+        """
+
+        # --interpret JSON file--
+        with open(jsonDescriptor, "r") as read_file:
+            jsonObject = json.load(read_file)
+
+
+    def object2json(self):
+        """
+               Function to generate a json file from the Material model
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class Contact():       #
+    """
+        Contact: Class representing the contact
+
+         :param string Name: Name of the Contact
+         :param string Type: Density of the Contact
+         :param Double SearchDistance: Search distance of the Contact
+         :param list ApplicationAreas: list of application areas
+
+
+    """
+
+    def __init__(self, JSONDescriptor=None, Name="", Type="",SearchDistance=5.0,ApplicationAreas=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Type = Type
+            self.SearchDistance = SearchDistance
+            self.ApplicationAreas = ApplicationAreas
+
+        else:
+            self.json2object(JSONDescriptor)
+            x = 10
+
+    def json2object(self, jsonDescriptor):
+        """
+             Function to generate a Contact object from a JSON file
+
+             :param string jsonDescriptor: absolute path to the json file for the Contact
+
+        """
+
+        # --interpret JSON file--
+        with open(jsonDescriptor, "r") as read_file:
+            jsonObject = json.load(read_file)
+
+
+    def object2json(self):
+        """
+               Function to generate a json file from the Contact model
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class BoundaryCondition():       #
+    """
+        BoundaryCondition: Class representing the BoundaryCondition
+
+         :param string Name: Name of the BoundaryCondition
+         :param string Type: Density of the BoundaryCondition
+         :param Double Value: Value of the BoundaryCondition
+         :param list Direction: Direction vector of BoundaryCondition
+         :param list DOF: DOF vector of BoundaryCondition
+         :param list ApplicationAreas: list of application areas
+
+
+    """
+
+    def __init__(self, JSONDescriptor=None, Name="", Type="",Value=0.0,Direction=[],DOF=[],ApplicationAreas=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Type = Type
+            self.Value = Value
+            self.DOF = DOF
+            self.Direction = Direction
+            self.ApplicationAreas = ApplicationAreas
+
+        else:
+            self.json2object(JSONDescriptor)
+            x = 10
+
+    def json2object(self, jsonDescriptor):
+        """
+             Function to generate a BoundaryCondition object from a JSON file
+
+             :param string jsonDescriptor: absolute path to the json file for the Contact
+
+        """
+
+        # --interpret JSON file--
+        with open(jsonDescriptor, "r") as read_file:
+            jsonObject = json.load(read_file)
+
+
+    def object2json(self):
+        """
+               Function to generate a json file from the BoundaryCondition
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class ResultMetric():       #
+    """
+        ResultMetric: Class representing a result metric
+
+         :param string Name: Name of the ResultMetric
+         :param string Description: Description of the ResultMetric
+         :param string Unit: Unit of the ResultMetric
+         :param double Optimum: Optimum value of the ResultMetric
+         :param double Value: Value of the ResultMetric
+
+
+    """
+
+    def __init__(self, JSONDescriptor=None, Name="", Description="",Optimum=5.0,Unit="",Value=0.0, DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Description = Description
+            self.Optimum = Optimum
+            self.Unit = Unit
+            self.Value = Value
+
+        else:
+            self.json2object(JSONDescriptor)
+            x = 10
+
+    def json2object(self, jsonDescriptor):
+        """
+             Function to generate a Contact object from a JSON file
+
+             :param string jsonDescriptor: absolute path to the json file for the Contact
+
+        """
+
+        # --interpret JSON file--
+        with open(jsonDescriptor, "r") as read_file:
+            jsonObject = json.load(read_file)
+
+
+    def object2json(self):
+        """
+               Function to generate a json file from the Contact model
+        """
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+class AnalysisModel():       #
+    """
+        AnalysisModel: Class representing the AnalysisModel
+
+         :param string Name: Name of the AnalysisModel
+         :param string Description: Description of the AnalysisModel
+         :param string GUID: Global unique identifier of the AnalysisModel
+         :param string Version: Version of the AnalysisModel
+         :param string ModelFile: ModelFile of the AnalysisModel
+         :param string AnalysisType: AnalysisType of the AnalysisModel
+         :param double SubsetNumber: SubsetNumber of the AnalysisModel
+         :param list ModelDescription: ModelDescription of AnalysisModel [Formalism,Version,CreatedBy]
+         :param list Mesh: MeshTypes of AnalysisModel [Meshtypes]
+         :param object list Contacts: List of contacts of AnalysisModel
+         :param object list BoundaryConditions: List of BoundaryConditions of AnalysisModel
+         :param object list DecisionVariables: List of DecisionVariables of AnalysisModel
+         :param list ApplicationAreas: list of application areas
+
+
+    """
+
+    def __init__(self, JSONDescriptor=None, Name="", Description="",Version="",GUID="",AnalysisType="",ModelFile="",SubsetNumber=0.0,ModelDescription=[],Mesh=[],Contacts=[],BoundaryConditions=[],DecisionVariables=[],ApplicationAreas=[], DEBUG=True):
+        self.DEBUG = DEBUG
+        if JSONDescriptor is None:
+            self.Name = Name
+            self.Description = Description
+            self.GUID=GUID
+            self.Version = Version
+
+            self.ModelFile=ModelFile
+            self.SubsetNumber=SubsetNumber
+            self.ModelDescription=ModelDescription
+            self.Mesh = Mesh
+            #analysis elements
+            self.AnalysisGUID = ""
+            self.AnalysisType=AnalysisType
+            self.ToolDescription = []
+            self.AnalysisResults = []
+            self.Responsepoints = []
+            self.Contacts = Contacts
+            self.BoundaryConditions = BoundaryConditions
+            self.DecisionVariables = DecisionVariables
+            self.ApplicationAreas = ApplicationAreas
+
+        else:
+            self.json2object(JSONDescriptor)
+            x = 10
+
+    def clean(self):
+        self.ModelDescription = []
+        self.Mesh = []
+        self.Contacts = []
+        self.BoundaryConditions = []
+        self.DecisionVariables = []
+        self.ApplicationAreas = []
+
+    def json2object(self, jsonDescriptor):
+        """
+             Function to generate a BoundaryCondition object from a JSON file
+
+             :param string jsonDescriptor: absolute path to the json file for the Contact
+
+        """
+
+        # --interpret JSON file--
+        with open(jsonDescriptor, "r") as read_file:
+            jsonObject = json.load(read_file)
+
+
+    def object2json(self):
+        """
+               Function to generate a json file from the BoundaryCondition
+        """
+        return json.dumps(self, default=lambda o: o.__dict__,indent=4)
+
 
 #---------------------Ontology & contracts interface objects--------------------------
 
@@ -924,4 +1318,6 @@ class DomainVariable():       #
         """
                Function to generate a json file from the DomainVariable model
         """
-        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
\ No newline at end of file
+        return json.dumps(self, default=lambda o: o.__dict__, indent=4)
+
+
