@@ -44,13 +44,14 @@ from .InterfaceObjects import ASG,StopCondition,DFARule,Product,Parameter,Produc
 
 class KB_Interface():
 
-    def __init__(self,DEBUG):
+    def __init__(self,DEBUG,KB_BASELINE='input/metamodel/Version8/PACoMM.ecore'):
         self.DEBUG = DEBUG
         self.MM = None
         self.model_instance = None
         self.model = None
         self.KB_path = ""
         self.ECORE_path = ""
+        self.KB_BASELINE = KB_BASELINE
 
 # -----------------------------------------------------------------------------------------------------------
 #
@@ -117,6 +118,48 @@ class KB_Interface():
         """
         setASG(AssemblySystemName, InterfaceObject,self.model,self.KB_path,self.ECORE_path,self.MM,self)
 
+    # -----------------------------------------------------------------------------------------------------------
+    #
+    #   ASG data extension
+    #
+    # -----------------------------------------------------------------------------------------------------------
+    def getASGAlgorithmData(self, AssemblySystemName):
+        """
+              Function to fetch the Assembly Sequence Generation (ASG) data using the name as identifier.
+              The function returns an interface block of the ASG
+
+              :param string AssemblySystemName: Name identifier of the assemlbySystem
+
+              :return object ASG data Model: Interface object of the ASG (-1 = error)
+        """
+        return getASGAlgorithmData(AssemblySystemName, self.model)
+
+    def updateASGAlgorithmData(self, AssemblySystemName, interfaceObject):
+        """
+              Function to update the  complete Assembly Sequence Generation (ASG) data using the name as identifier.
+              The ASG interface object is used to interface with the function.
+              The function returns whether or not the function is performed correctly
+
+              :param string AssemblySystemName: Name identifier of the assemlbySystem
+              :param object ASG data Model: Interface object of the ASG data
+
+              :return int Error: -1 = error, 1= function performed correcly
+        """
+
+        return updateASGAlgorithmData(AssemblySystemName, interfaceObject, self.model, self.KB_path)
+
+    def setASGAlgorithmData(self, AssemblySystemName, InterfaceObject):
+        """
+              Function to add a complete ASG data model to an blank KB (no existing ASG part).
+              The function returns whether or not the function is performed correctly (NOT YET)
+
+              :param string AssemblySystemName: Name identifier of the assemlbySystem
+              :param object ASG data Model: Interface object of the ASG data Model model
+
+              :return int Error: -1 = error, 1= function performed correcly
+        """
+        setASGAlgorithmData(AssemblySystemName, InterfaceObject, self.model, self.KB_path, self.ECORE_path, self.MM, self)
+
 # -----------------------------------------------------------------------------------------------------------
 #
 #   DFA extension
@@ -236,6 +279,46 @@ class KB_Interface():
         """
         return setPerformance(interfaceObject,self.model,self.KB_path,self)
 
+    # -----------------------------------------------------------------------------------------------------------
+    #
+    #   Analysis extension
+    #
+    # -----------------------------------------------------------------------------------------------------------
+
+    def getAnalysisModel(self, analysisModelName):
+        """
+          Function to fetch the complete Analysis Model using the name as identifier.
+          The function returns an interface block of the performance model
+
+          :param string analysisModelName: Name identifier of the optimization problem
+
+          :return object analysisModelName: Interface object of the analysis Model (-1 = error)
+        """
+        return getAnalysisModel(analysisModelName, self.model)
+
+    def updateAnalysisModel(self, interfaceObject):
+        """
+              Function to update the complete Analysis Model matching the name as identifier.
+              The performance interface object is used to interface with the function.
+              The function returns whether or not the function is performed correctly
+
+              :param object analysisModel: Interface object of the analysis Model
+
+              :return int Error: -1 = error, 1= function performed correcly
+        """
+        return updateAnalysisModel(interfaceObject, self.model, self.KB_path)
+
+    def setAnalysisModel(self, interfaceObject):
+        """
+             Function to add a complete Analysis Model to an blank KB (no existing performance part).
+             The performance interface object is used to interface with the function.
+             The function returns whether or not the function is performed correctly
+
+             :param object analysisModel: Interface object of the performance model
+
+             :return int Error: -1 = error, 1= function performed correcly
+        """
+        return setAnalysisModel(interfaceObject, self.model, self.KB_path, self)
 
 
 # -----------------------------------------------------------------------------------------------------------
@@ -327,12 +410,12 @@ class KB_Interface():
         if len(self.model.includesProductPart.items) != 0:
             for productPart in self.model.includesProductPart.items:
                 if ProductPartName == productPart.hasPartName:
-                    part = ProductPart(Name=productPart.hasPartName,ProductType=productPart.hasPartType,Quantity=1)        #TODO:Quantity not in MM v6
+                    part = ProductPart(Name=productPart.hasPartName,ProductType=productPart.hasPartType,Quantity=1,isFastner=productPart.isFastner)        #TODO:Quantity not in MM v6
                     #link the parameters
                     part.parameterList = []
                     try:
                         for par in productPart.hasModel.hasModelParameter.items:
-                            p = Parameter(Name='Not defined',Key=par.hasKey,Value=par.hasValue,Stopcriteria='Not defined')  #TODO: no complete parameter description is given!
+                            p = Parameter(Name='Not defined',Key=par.hasKey,Value=par.hasValue,Stopcriteria='Not defined',Unit=par.hasUnit)  #TODO: no complete parameter description is given!
                             part.parameterList.append(p)
                     except:
                         if self.DEBUG:print("No model added to ProductPart")
@@ -374,6 +457,7 @@ class KB_Interface():
                     #productPart.hasDescription = interfaceObject.Description       # TODO:Quantity not in MM v6
                     productPart.hasPartType=interfaceObject.ProductType
                     #productPart.Quantity = interfaceObject.Quantity                 # TODO:Quantity not in MM v6
+                    productPart.isFastner = interfaceObject.isFastner
 
                     # update the parameters
                     for par in productPart.hasModel.hasModelParameter.items:
@@ -381,6 +465,7 @@ class KB_Interface():
                             if par_new.Key == par.hasKey:
                                 par.hasKey = par_new.Key
                                 par.hasValue = par_new.Value
+                                par.hasUnit = par_new.Unit
 
 
                     # link the product parts - Recursive!
@@ -426,6 +511,7 @@ class KB_Interface():
         # productPart.hasDescription = interfaceObject.Description       # TODO:Quantity not in MM v6
         part.hasPartType = interfaceObject.ProductType
         # productPart.Quantity = interfaceObject.Quantity                 # TODO:Quantity not in MM v6
+        part.isFastner = interfaceObject.isFastner
 
 
         #add model to ProductPart
@@ -438,6 +524,9 @@ class KB_Interface():
             parameter = self.create_connected("Parameter")
             parameter.hasKey = par_new.Key
             parameter.hasValue = par_new.Value
+            parameter.hasName = par_new.Name
+            parameter.hasValue = par_new.Value
+            parameter.hasUnit = par_new.Unit
             part.hasModel.hasModelParameter.append(parameter)
 
         # link the product parts - Recursive! # TODO:update the subproduct - No example available to validate
@@ -894,6 +983,17 @@ class KB_Interface():
         self.MM,self.model,self.model_instance = importInstanceModel_NEW(path_ecore, path_instance, VERBOSE=False)
         return self.MM,self.model,self.model_instance
 
+    def importKBInstanceModel(self, path_instance, VERBOSE=False):
+        """
+              For internal use only!
+            """
+        # LOAD THE BASELINE KB ECORE FILE
+        path_ecore = self.resolvePath(self.KB_BASELINE)
+        self.KB_path = path_instance  # To update current KB
+        self.ECORE_path = path_ecore
+        self.MM,self.model,self.model_instance = importInstanceModel_NEW(path_ecore, path_instance, VERBOSE=False)
+        return self.MM,self.model,self.model_instance
+
     def addXMLResource(self, rset):
         """
               For internal use only!
@@ -1043,6 +1143,7 @@ class KB_Interface():
             p.hasDescription = par[1]
             p.hasKey = par[2]
             p.hasValue = par[3]
+            p.hasUnit = par[4]
             model.hasModelParameter.items.append(p)
 
         return model
